/**
 * Created by amin on 5/3/16.
 * Modified by emran on 10/19/16.
 */
"use strict";
const WebSocket = require("ws");
const ApiConst_1 = require("../../api/ApiConst");
const Logger_1 = require("../../utils/Logger");
const BotError_1 = require("../../utils/BotError");
class SocketConnection {
    /**
     * construct class with botToken
     * @param botToken
     * @param callback
     */
    constructor(botToken, callback) {
        this.shouldReconnect = false;
        this._isOnline = false;
        this.trying = false;
        this.sendPostponeCounter = 0;
        this.shouldLogConnecting = true;
        this.socketOpened = false;
        this.gotSocketResponse = false;
        this.botToken = botToken;
        this.onReceiveCallBack = callback;
        this.trying = true;
        this.connect();
        this.checkIsOnline();
        this.startReconnectService();
    }
    checkIsOnline() {
        let sent = false;
        let noResponseCount = 0;
        let This = this;
        let f = function () {
            if (!sent) {
                if (This.sendPostponeCounter === 8) {
                    This.sendPostponeCounter = 0;
                    try {
                        This.gotSocketResponse = false;
                        This.socket.send('{"$type":"Request","id":"1","service":"users","body":{"$type":"IsAdmin","userId":1429755504}}');
                        sent = true;
                    }
                    catch (e) {
                        console.log("Socket Failed");
                        This._isOnline = false;
                        This.shouldReconnect = true;
                    }
                }
                else
                    This.sendPostponeCounter++;
            }
            else {
                if (!This.gotSocketResponse) {
                    noResponseCount++;
                    Logger_1.Logger.debug("No response #" + noResponseCount);
                    if (noResponseCount == 4) {
                        //if (!SocketConnection.isOnline) {
                        console.log("No Connection!");
                        This._isOnline = false;
                        This.shouldReconnect = true;
                    }
                }
                else {
                    // Connection is ok. prepare for the next send try:
                    This._isOnline = true;
                    sent = false;
                    noResponseCount = 0;
                }
            }
            setTimeout(f, 5000);
        };
        f();
    }
    startReconnectService() {
        let This = this;
        let f = () => {
            if (This.shouldReconnect && !This.trying) {
                try {
                    This.socket.pause();
                }
                catch (e) {
                }
                try {
                    This.socket.close();
                }
                catch (e) {
                }
                This.socket.removeListener('close', This.onClose);
                This.socket.removeListener('open', This.onError);
                This.socket.removeListener('error', This.onError);
                This.trying = true;
                This.shouldReconnect = false;
                This.connect();
            }
            setTimeout(f, 1000);
        };
        f();
    }
    /**
     * connect to server and set callback for handling Errors
     * @returns {Promise<any>}
     * */
    connect() {
        if (this.shouldLogConnecting) {
            Logger_1.Logger.log('Socket Connecting...');
            this.shouldLogConnecting = false;
        }
        let This = this;
        this.socket = new WebSocket(ApiConst_1.ApiConst.BASE_API_URL + this.botToken);
        this.socket.on('open', () => this.onOpen.call(This));
        this.socket.on('error', (err) => This.onError.call(This, err));
        this.socket.on('close', (code, message) => This.onClose.call(This, code, message));
        return this.connectPromise = new Promise((resolve, reject) => {
            this.connectResolve = resolve;
        });
    }
    onOpen() {
        Logger_1.Logger.log('Socket Connected!');
        this.shouldLogConnecting = true;
        this.socketOpened = true;
        this.gotSocketResponse = true;
        this._isOnline = true;
        this.trying = false;
        let This = this;
        this.socket.on('message', (data) => {
            This.checkSocketResponse(data);
            This.onReceiveCallBack.call(This, data);
        });
        this.connectResolve();
    }
    ;
    onError(err) {
        let error = new BotError_1.BotError(err);
        this.trying = false;
        if (this.socketOpened)
            Logger_1.Logger.error("Socket error cause: " + error.description);
    }
    ;
    onClose(code, message) {
        let error = new BotError_1.BotError(new Error(code + " " + message));
        if (this.socketOpened) {
            Logger_1.Logger.log('Connection Failed');
            Logger_1.Logger.error("Socket close cause: " + error.description);
            this.socketOpened = false;
        }
        this.trying = false;
        this.shouldReconnect = true;
    }
    ;
    checkSocketResponse(jsonPacket) {
        this.sendPostponeCounter = 0;
        let obj = JSON.parse(jsonPacket);
        if (obj.id != '1')
            return;
        Logger_1.Logger.debug("Socket connection successfully checked.");
        this.gotSocketResponse = true;
    }
    /**
     *
     * @param jsonString string of message json
     * if connection is stable send message and return true else return false
     * @returns {Promise<boolean>}
     */
    send(jsonString) {
        try {
            this.socket.send(jsonString);
            return Promise.resolve(true);
        }
        catch (e) {
            if (this._isOnline)
                this.shouldReconnect = true;
            return Promise.resolve(false);
        }
    }
    /**
     * set callback to on message receive from socket
     * @param callback on message recived callback
     * @return {void}
     */
    setOnReceive(callback) {
        this.onReceiveCallBack = callback;
        //SocketConnection.socket.on('message', (data) => callback(data));
    }
    /**
     * Determines whether the connection to the server is established.
     * @return {number} True if is connected; Otherwise false.
     */
    isOnline() {
        return this._isOnline;
    }
}
exports.SocketConnection = SocketConnection;
//# sourceMappingURL=SocketConnection.js.map